@@ -1,11 +1,12 @@
 use super::bed;
 use core::{fmt, panic};
+use itertools::Itertools;
 use lazy_static::lazy_static;
 use regex::Regex;
 use rust_htslib::bam::record::Cigar::*;
 use rust_htslib::bam::record::CigarString;
 use rust_htslib::bam::record::*;
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::fs;
 use std::io;
 use std::io::BufRead;
@@ -70,6 +71,124 @@ impl<'a> Paf<'a> {
         paf.records = records;
         paf
     }
+
+    pub fn filter_aln_pairs(&mut self, paired_len: u64) {
+        let mut dict = HashMap::new();
+        for rec in self.records.iter_mut() {
+            let aln_bp = dict
+                .entry((rec.t_name.clone(), rec.q_name.clone()))
+                .or_insert(0_u64);
+            *aln_bp += rec.t_en - rec.t_st;
+        }
+        self.records.retain(|rec| {
+            paired_len < *dict.get(&(rec.t_name.clone(), rec.q_name.clone())).unwrap()
+        });
+    }
+
+    pub fn filter_query_len(&mut self, min_query_len: u64) {
+        self.records.retain(|rec| rec.q_len > min_query_len);
+    }
+
+    /// Filter on alignment length
+    pub fn filter_aln_len(&mut self, min_aln_len: u64) {
+        self.records.retain(|rec| rec.t_en - rec.t_st > min_aln_len);
+    }
+
+    /// orient queries relative to their target (inverts if most bases are aligned rc).
+    pub fn orient(&mut self) {
+        let mut orient_order_dict = HashMap::new();
+        let mut t_names = HashSet::new();
+        // calculate whether a contig is mostly forward or reverse strand
+        // and determine the middle alignment position with respect to the target
+        for rec in &self.records {
+            let (orient, total_bp, order) = orient_order_dict
+                .entry((rec.t_name.clone(), rec.q_name.clone()))
+                .or_insert((0_i64, 0_u64, 0_u64));
+            // set the orientation of the query relative to the target
+            if rec.strand == '-' {
+                *orient -= (rec.q_en - rec.q_st) as i64;
+            } else {
+                *orient += (rec.q_en - rec.q_st) as i64;
+            }
+            // set a number that will determine the order of the contig
+            let weight = rec.t_en - rec.t_st;
+            *total_bp += weight;
+            *order += weight * (rec.t_st + rec.t_en) / 2;
+            // make a list of targets
+            t_names.insert(rec.t_name.clone());
+        }
+
+        // set the order and orientation of records
+        for rec in &mut self.records {
+            // set the order of the records
+            let (orient, total_bp, order) = orient_order_dict
+                .get(&(rec.t_name.clone(), rec.q_name.clone()))
+                .unwrap();
+            rec.order = *order / *total_bp;
+
+            // reverse record if it is mostly on the rc
+            if *orient < 0 {
+                rec.q_name = format!("{}-", rec.q_name);
+                let new_st = rec.q_len - rec.q_en;
+                let new_en = rec.q_len - rec.q_st;
+                rec.q_st = new_st;
+                rec.q_en = new_en;
+                rec.strand = if rec.strand == '+' { '-' } else { '+' };
+            } else {
+                rec.q_name = format!("{}+", rec.q_name);
+            }
+        }
+    }
+
+    // scaffold oriented contigs into one fake super contig
+    pub fn scaffold(&mut self, spacer_size: u64) {
+        // sort the records by their target name and order
+        self.records.sort_by(|a, b| {
+            a.t_name
+                .cmp(&b.t_name) // group by target
+                .then(a.order.cmp(&b.order)) // order query by position in target
+                .then(a.q_st.cmp(&b.q_st)) // order by position in query
+        });
+
+        // group by t_name
+        for (_t_name, t_recs) in &self.records.iter_mut().group_by(|rec| rec.t_name.clone()) {
+            let mut t_recs: Vec<&mut PafRecord> = t_recs.collect();
+            // sort recs by order
+            t_recs.sort_by(|a, b| {
+                a.order
+                    .cmp(&b.order) // order query by position in target
+                    .then(a.q_st.cmp(&b.q_st)) // order by position in query
+            });
+
+            // new scaffold name
+            let scaffold_name = t_recs
+                .iter()
+                .map(|rec| rec.q_name.clone())
+                .unique()
+                .collect::<Vec<String>>()
+                .join("::");
+
+            let mut scaffold_len = 0_u64;
+            for (_q_name, q_recs) in &t_recs.iter_mut().group_by(|rec| rec.q_name.clone()) {
+                let q_recs: Vec<&mut &mut PafRecord> = q_recs.collect();
+                let q_min = q_recs.iter().map(|rec| rec.q_st).min().unwrap_or(0);
+                let q_max = q_recs.iter().map(|rec| rec.q_en).max().unwrap_or(0);
+                let added_q_bases = q_max - q_min;
+                for rec in q_recs {
+                    rec.q_st = rec.q_st - q_min + scaffold_len;
+                    rec.q_en = rec.q_en - q_min + scaffold_len;
+                }
+                scaffold_len += added_q_bases + spacer_size;
+            }
+            // remove padding insert on the end of rec
+            scaffold_len -= spacer_size;
+
+            for rec in t_recs {
+                rec.q_name = scaffold_name.clone();
+                rec.q_len = scaffold_len;
+            }
+        }
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -92,6 +211,7 @@ pub struct PafRecord {
     pub qpos_aln: Vec<u64>,
     pub long_cigar: CigarString,
     pub id: String,
+    pub order: u64,
 }
 
 impl PafRecord {
@@ -147,6 +267,7 @@ impl PafRecord {
             qpos_aln: Vec::new(),
             long_cigar: CigarString(Vec::new()),
             id: "".to_string(),
+            order: 0,
         };
         Ok(rec)
     }
@@ -171,6 +292,7 @@ impl PafRecord {
             qpos_aln: Vec::new(),
             long_cigar: CigarString(Vec::new()),
             id: self.id.clone(),
+            order: self.order,
         }
     }
 
